/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "quakedef.h"

// these are initialized to invalid values so they will update correctly first time they're seen
static int r_blendstate = -1;
static int r_depthstate = -1;

// this must be 0
static int r_currentarrays = 0;

static int r_currenttmu = 0;
static int r_currenttexture[16];

void GL_BlendState (int bs)
{
	if (bs == r_blendstate) return;

	switch (bs)
	{
	case BS_NONE:
	default:
		glDisable (GL_BLEND);
		break;

	case BS_ALPHABLEND:
		glEnable (GL_BLEND);
		glBlendEquation (GL_FUNC_ADD);
		glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		break;

	case BS_ADDITIVE:
		glEnable (GL_BLEND);
		glBlendEquation (GL_FUNC_ADD);
		glBlendFunc (GL_ONE, GL_ONE);
		break;

	case BS_MAXBLEND:
		glEnable (GL_BLEND);
		glBlendEquation (GL_MAX);
		glBlendFunc (GL_ONE, GL_ONE);
		break;
	}

	r_blendstate = bs;
}


void GL_DepthState (int ds)
{
	if (ds == r_depthstate) return;

	// testing
	if (ds & DS_TESTLESSEQUAL)
	{
		glEnable (GL_DEPTH_TEST);
		glDepthFunc (GL_LEQUAL);
	}
	else if (ds & DS_TESTEQUAL)
	{
		glEnable (GL_DEPTH_TEST);
		glDepthFunc (GL_EQUAL);
	}
	else glDisable (GL_DEPTH_TEST);

	// writing
	if (ds & DS_WRITE)
		glDepthMask (GL_TRUE);
	else
		glDepthMask (GL_FALSE);

	r_depthstate = ds;
}


void GL_EnableVertexAttribArrays (int arrays)
{
	int i;

	if (arrays == r_currentarrays) return;

	for (i = 0; i < 16; i++)
	{
		if ((arrays & (1 << i)) && !(r_currentarrays & (1 << i))) glEnableVertexAttribArray (i);
		if (!(arrays & (1 << i)) && (r_currentarrays & (1 << i))) glDisableVertexAttribArray (i);
	}

	r_currentarrays = arrays;
}


void GL_BindTexture (int unit, int texnum)
{
	if (r_currenttexture[unit] == texnum)
		return;

	if (r_currenttmu != unit)
	{
		glActiveTexture (GL_TEXTURE0 + unit);
		r_currenttmu = unit;
	}

	glBindTexture(GL_TEXTURE_2D, texnum);
	r_currenttexture[unit] = texnum;
}


