/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// gl_warp.c -- sky and water polygons

#include "quakedef.h"

int		solidskytexture;
int		alphaskytexture;


void R_SetupWarpShaderParams (void)
{
	float warptime = cl.time - ((M_PI * 2) * (int) (cl.time / (M_PI * 2)));
	glmatrix warpmatrix = {M_PI, 0, 0, 0, 0, M_PI, 0, 0, 0, 0, 1, 0, warptime, warptime, 0, 1};

	glMatrixMode (GL_MATRIX1_ARB);
	glLoadMatrixf (warpmatrix.m16);
	glMatrixMode (GL_MODELVIEW);
}


void R_DrawWaterChain (msurface_t *surf)
{
	texture_t *lasttex = NULL;

	GL_BindPrograms (r_warpsurfvp, r_warpsurffp);
	GL_BlendState (BS_NONE);
	GL_DepthState (DS_WRITE | DS_TESTLESSEQUAL);

	for (; surf; surf = surf->texturechain)
	{
		if (surf->texinfo->texture != lasttex)
		{
			GL_EndDrawingSurfaces ();
			GL_BindTexture (0, surf->texinfo->texture->gl_texturenum);
			lasttex = surf->texinfo->texture;
		}

		GL_DrawSurface (surf);
	}

	GL_EndDrawingSurfaces ();
}


void R_SetupSkyShaderParams (void)
{
	float speedscale[2] = {cl.time * 8.0f, cl.time * 16.0f};
	glmatrix skymatrix = {1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 3, 0, -r_refdef.vieworigin[0], -r_refdef.vieworigin[1], -r_refdef.vieworigin[2] * 3.0f, 1};

	glProgramEnvParameter4fARB (GL_FRAGMENT_PROGRAM_ARB,
		4,
		(speedscale[0] - ((int) speedscale[0] & ~127)) * 0.0078125f,
		(speedscale[1] - ((int) speedscale[1] & ~127)) * 0.0078125f,
		0,
		0);

	glMatrixMode (GL_MATRIX0_ARB);
	glLoadMatrixf (skymatrix.m16);
	glMatrixMode (GL_MODELVIEW);
}


void R_DrawSkyChain (msurface_t *surf)
{
	GL_BindTexture (0, solidskytexture);
	GL_BindTexture (1, alphaskytexture);
	GL_BindPrograms (r_skysurfvp, r_skysurffp);

	GL_BlendState (BS_NONE);
	GL_DepthState (DS_WRITE | DS_TESTLESSEQUAL);

	for (; surf; surf = surf->texturechain)
		GL_DrawSurface (surf);

	GL_EndDrawingSurfaces ();
}


/*
=============
R_InitSky

A sky texture is 256*128, with the right side being a masked overlay
==============
*/
void R_InitSky (byte *src)
{
	int			i, j, p;
	unsigned	trans[128 * 128];
	unsigned	transpix;
	int			r, g, b;
	unsigned	*rgba;

	// make an average value for the back to avoid
	// a fringe on the top level
	r = g = b = 0;

	for (i = 0; i < 128; i++)
	{
		for (j = 0; j < 128; j++)
		{
			p = src[i * 256 + j + 128];
			rgba = &d_8to24table_rgba[p];
			trans[(i * 128) + j] = *rgba;
			r += ((byte *) rgba)[0];
			g += ((byte *) rgba)[1];
			b += ((byte *) rgba)[2];
		}
	}

	((byte *) &transpix)[0] = r / (128 * 128);
	((byte *) &transpix)[1] = g / (128 * 128);
	((byte *) &transpix)[2] = b / (128 * 128);
	((byte *) &transpix)[3] = 0;

	if (!solidskytexture)
		solidskytexture = texture_extension_number++;
	GL_BindTexture (0, solidskytexture);
	glTexImage2D (GL_TEXTURE_2D, 0, gl_solid_format, 128, 128, 0, GL_RGBA, GL_UNSIGNED_BYTE, trans);
	glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	for (i = 0; i < 128; i++)
	{
		for (j = 0; j < 128; j++)
		{
			p = src[i * 256 + j];
			if (p == 0)
				trans[(i * 128) + j] = transpix;
			else
				trans[(i * 128) + j] = d_8to24table_rgba[p];
		}
	}

	if (!alphaskytexture)
		alphaskytexture = texture_extension_number++;
	GL_BindTexture (0, alphaskytexture);
	glTexImage2D (GL_TEXTURE_2D, 0, gl_alpha_format, 128, 128, 0, GL_RGBA, GL_UNSIGNED_BYTE, trans);
	glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
}

